@@ -1,5 +1,7 @@
 /* Released under GPLv2 with exception for the OpenSSL library. See license.txt */
+/* $Revision$ */
 
+#include <assert.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -9,16 +11,15 @@
 #endif
 
 #include "gen.h"
-#include "mem.h"
 #include "http.h"
 #include "io.h"
-#include "str.h"
 #include "utils.h"
 
-int get_HTTP_headers(int socket_h, SSL *ssl_h, char **headers, int *overflow, int timeout)
+int get_HTTP_headers(int socket_h, SSL *ssl_h, char **headers, int *overflow, double timeout)
 {
+	char *term = NULL;
 	int len_in=0, len=4096;
-	char *buffer = mymalloc(len, "http header");
+	char *buffer = (char *)malloc(len + 1);
 	int rc = RC_OK;
 
 	*headers = NULL;
@@ -27,8 +28,8 @@ int get_HTTP_headers(int socket_h, SSL *ssl_h, char **headers, int *overflow, in
 
 	for(;;)
 	{
-		int rrc;
-		int now_n = (len - len_in) - 1;
+		int rrc = -1;
+		int now_n = len - len_in;
 
 #ifndef NO_SSL
 		if (ssl_h)
@@ -36,6 +37,57 @@ int get_HTTP_headers(int socket_h, SSL *ssl_h, char **headers, int *overflow, in
 		else
 #endif
 			rrc = read_to(socket_h, &buffer[len_in], now_n, timeout);
+
+		if (rrc == 0 || rrc == RC_SHORTREAD)	/* socket closed before request was read? */
+		{
+			rc = RC_SHORTREAD;
+			break;
+		}
+		else if (rrc < 0)
+		{
+			free(buffer);
+			return rrc;
+		}
+
+		len_in += rrc;
+
+		assert(len_in >= 0);
+		assert(len_in <= len);
+
+		buffer[len_in] = 0x00;
+
+		if (strstr(buffer, "\r\n\r\n") != NULL)
+			break;
+
+		if (len_in >= len)
+		{
+			len <<= 1;
+			buffer = (char *)realloc(buffer, len + 1);
+		}
+	}
+
+	*headers = buffer;
+
+	term = strstr(buffer, "\r\n\r\n");
+	if (term)
+		*overflow = len_in - (term - buffer + 4);
+	else
+		*overflow = 0;
+
+	return rc;
+}
+
+int dumb_get_HTTP_headers(int socket_h, char **headers, double timeout)
+{
+	int len_in=0, len=4096;
+	char *buffer = (char *)malloc(len);
+	int rc = RC_OK;
+
+	*headers = NULL;
+
+	for(;;)
+	{
+		int rrc = read_to(socket_h, &buffer[len_in], 1, timeout);
 		if (rrc == 0 || rrc == RC_SHORTREAD)	/* socket closed before request was read? */
 		{
 			rc = RC_SHORTREAD;
@@ -50,23 +102,19 @@ int get_HTTP_headers(int socket_h, SSL *ssl_h, char **headers, int *overflow, in
 		len_in += rrc;
 
 		buffer[len_in] = 0x00;
-		if (strstr(buffer, "\r\n\r\n") != NULL)
+		if (memcmp(&buffer[len_in - 4], "\r\n\r\n", 4) == 0)
+			break;
+		if (memcmp(&buffer[len_in - 2], "\n\n", 2) == 0) /* broken proxies */
 			break;
 
 		if (len_in == (len - 1))
 		{
 			len <<= 1;
-			buffer = (char *)myrealloc(buffer, len, "http reply");
+			buffer = (char *)realloc(buffer, len);
 		}
 	}
 
 	*headers = buffer;
 
-	char *term = strstr(buffer, "\r\n\r\n");
-	if (term)
-		*overflow = len_in - (term - buffer + 4);
-	else
-		*overflow = 0;
-
 	return rc;
 }
